# RealTimeCrypto
# Copyright 2024 SanderSita
# See LICENSE for details.

"""
RealTimeCrypto
"""

__version__ = "0.0.1"
__author__ = "SanderSita"
__license__ = "MIT"

from .api import RealTimeCrypto
from .models import (
    Coin,
    CryptoWebsocket,
    Statistic,
    WebsocketDetails,
    Top100Coin,
    HistoryPoint,
    GreedFearHistoryPoint,
    PerformingCrypto,
)
